﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MotorBoat;
using MotorBoat.DrawningObjects;
using MotorBoat.Entities;

namespace MotorBoat.DrawningObjects
{
    /// <summary>.
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningMotorBoat : DrawningBoat
    {
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="glass">Признак наличия стекла спереди</param>
        /// <param name="engine">Признак наличия двигателя</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public DrawningMotorBoat(int speed, double weight, Color bodyColor, Color additionalColor, bool glass, 
            bool engine, int width, int height) : base (speed, weight, bodyColor, width, height, 165, 80)
        {
            if (EntityBoat != null)
            {
                EntityBoat = new EntityMotorBoat(speed, weight, bodyColor, additionalColor, glass, engine);
            }
        }

        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public override void DrawTransport(Graphics g)
        {
            if (EntityBoat is not EntityMotorBoat motorBoat)
            {
                return;
            }

            Pen pen = new(Color.Black, 1);
            Brush additionalBrush = new SolidBrush(motorBoat.AdditionalColor);

            base.DrawTransport(g);

            if (motorBoat.Glass)
            {
                // стекло впереди
                Brush glassBrush = new SolidBrush(Color.LightBlue);
                Point[] glassPoints = new Point[]
                {
                new Point(_startPosX + 100, _startPosY + 15),
                new Point(_startPosX + 120, _startPosY + 25),
                new Point(_startPosX + 120, _startPosY + 45),
                new Point(_startPosX + 100, _startPosY + 55)
                };
                g.FillPolygon(glassBrush, glassPoints);
                g.DrawPolygon(pen, glassPoints);
            }

            if (motorBoat.Engine)
            {
                // двигатель
                g.FillRectangle(additionalBrush, _startPosX + 0, _startPosY + 10, 5, 50);
                g.DrawRectangle(pen, _startPosX + 0, _startPosY + 10, 5, 50);
            }
        }
    }
}