﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using MotorBoat.DrawningObjects;
using MotorBoat.Generics;
using MotorBoat.MovementStrategy;

namespace MotorBoat
{
    /// <summary>
    /// Форма для работы с набором объектов класса DrawningBoat
    /// </summary>
    public partial class FormBoatCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly BoatsGenericStorage _storage;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBoatCollection()
        {
            InitializeComponent();
            _storage = new BoatsGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
        }

        /// <summary>
        /// Заполнение listBoxStorages
        /// </summary>
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;
            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i]);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 && index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }
        }

        /// <summary>
        /// Добавление набора в коллекцию
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAddObject_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
        }

        /// <summary>
        /// Выбор набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListBoxObjects_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image =
            _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowBoats();
        }

        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить объект {listBoxStorages.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(listBoxStorages.SelectedItem.ToString() ?? string.Empty);
                ReloadObjects();
            }
        }

        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddBoat_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
                return;

            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];

            if (obj == null)
                return;

            var FormBoatConfig = new FormBoatConfig();
            FormBoatConfig.AddEvent(AddBoat);
            FormBoatConfig.Show();
        }

        private void AddBoat(DrawningBoat drawningBoat)
        {
            if (listBoxStorages.SelectedIndex == -1)
                return;

            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];

            if (obj == null)
                return;

            if (obj + drawningBoat)
            {
                MessageBox.Show("Объект добавлен");
                pictureBoxCollection.Image = obj.ShowBoats();
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }
        }

        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveBoat_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
                return;

            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];

            if (obj == null)
                return;

            if (MessageBox.Show("Удалить объект?", "Удаление",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
            if (obj - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = obj.ShowBoats();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }

        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
                return;

            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];

            if (obj == null)
                return;

            pictureBoxCollection.Image = obj.ShowBoats();
        }
    }
}