﻿using MotorBoat.DrawningObjects;
using MotorBoat.Entities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MotorBoat
{
    public partial class FormBoatConfig : Form
    {
        /// <summary>
        /// Переменная-выбранная лодка
        /// </summary>
        DrawningBoat? _boat = null;

        /// <summary>
        /// Событие
        /// </summary>
        public event Action<DrawningBoat>? EventAddBoat;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBoatConfig()
        {
            InitializeComponent();
            panelBrown.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelMilky.MouseDown += PanelColor_MouseDown;
            panelPeach.MouseDown += PanelColor_MouseDown;
            panelGrey.MouseDown += PanelColor_MouseDown;
            panelOrange.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelCherry.MouseDown += PanelColor_MouseDown;

            buttonCancel.Click += (s, e) => Close();
        }

        /// <summary>
        /// Отрисовать лодку
        /// </summary>
        private void DrawBoat()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _boat?.SetPosition(5, 5);
            _boat?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }

        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void labelSimpleObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name, DragDropEffects.Move | DragDropEffects.Copy);
        }

        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void labelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void labelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _boat = new DrawningBoat((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _boat = new DrawningMotorBoat((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxGlass.Checked,
                   checkBoxEngine.Checked, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
            }
            DrawBoat();
        }

        /// <summary>
        /// Добавление лодки
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }

        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        internal void AddEvent(Action<DrawningBoat> ev)
        {
            if (EventAddBoat == null)
            {
                EventAddBoat = ev;
            }
            else
            {
                EventAddBoat += ev;
            }
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            EventAddBoat?.Invoke(_boat);
            Close();
        }

        private void labelBodyColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_boat == null)
                return;
            _boat?.EntityBoat?.setBodyColor((Color)e.Data.GetData(typeof(Color)));
            DrawBoat();
        }

        private void labelAdditionalColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_boat == null)
                return;
            if (!(_boat is DrawningMotorBoat))
                return;
            (_boat.EntityBoat as EntityMotorBoat)?.setAdditionalColor(color: (Color)e.Data.GetData(typeof(Color)));
            DrawBoat();
        }

        private void labelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }
}