using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using MotorBoat;
using MotorBoat.MovementStrategy;
using MotorBoat.DrawningObjects;

namespace MotorBoat
{
    public partial class FormMotorBoat : Form
    {
        /// <summary>
        /// -   
        /// </summary>
        private DrawningBoat? _drawningBoat;

        /// <summary>
        ///  
        /// </summary>
        private AbstractStrategy? _strategy;

        /// <summary>
        ///  
        /// </summary>
        public DrawningBoat? SelectedBoat { get; private set; }

        /// <summary>
        ///  
        /// </summary>
        public FormMotorBoat()
        {
            InitializeComponent();
            _strategy = null;
            SelectedBoat = null;
        }

        /// <summary>
        ///   
        /// </summary>
        private void Draw()
        {
            if (_drawningBoat == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxMotorBoat.Width, pictureBoxMotorBoat.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningBoat.DrawTransport(gr);
            pictureBoxMotorBoat.Image = bmp;
        }

        /// <summary>
        ///    "  "
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButonCreateMotorBoat_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color bodyColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new ColorDialog();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                bodyColor = dialog.Color;
            }
            Color additionalColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                additionalColor = dialog.Color;
            }
            _drawningBoat = new DrawningMotorBoat(random.Next(100, 300), random.Next(1000, 3000),
                bodyColor, additionalColor,
                Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)),
                pictureBoxMotorBoat.Width, pictureBoxMotorBoat.Height);
            _drawningBoat.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        /// <summary>
        ///    " "
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreate_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _drawningBoat = new DrawningBoat(random.Next(100, 300), random.Next(1000, 3000),
                color, pictureBoxMotorBoat.Width, pictureBoxMotorBoat.Height);
            _drawningBoat.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        /// <summary>
        ///   
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawningBoat == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningBoat.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawningBoat.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawningBoat.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawningBoat.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();
        }

        /// <summary>
        ///    ""
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonStep_Click(object sender, EventArgs e)
        {
            if (_drawningBoat == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(new
                DrawningObjectBoat(_drawningBoat), pictureBoxMotorBoat.Width,
                pictureBoxMotorBoat.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_strategy == null)
            {
                return;
            }
            _strategy.MakeStep();
            Draw();
            if (_strategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }

        /// <summary>
        ///  
        /// </summary>
        /// <param name="sender"></param>
        /// <<param name="e"></param>
        private void ButtonSelectBoat_Click(object sender, EventArgs e)
        {
            SelectedBoat = _drawningBoat;
            DialogResult = DialogResult.OK;
        }
    }
}